﻿namespace Prism.Sample.Shell.ViewModels
{
    using System.ComponentModel;
    using System.Linq;
    using System.Windows.Input;
    using Microsoft.Practices.Prism.Commands;
    using Microsoft.Practices.Prism.MefExtensions.Modularity;
    using Microsoft.Practices.Prism.Modularity;
    using Microsoft.Practices.ServiceLocation;
    using Prism.Sample.Infrastructure;

    /// <summary>
    /// Shell ViewModel.
    /// </summary>
    public class ShellViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Load Module Command.
        /// </summary>
        private DelegateCommand<object> loadModuleCommand;

        /// <summary>
        /// Prism Module Manager.
        /// </summary>
        private IModuleManager moduleManager;

        /// <summary>
        /// Prism Module Catalog.
        /// </summary>
        private IModuleCatalog moduleCatalog;

        /// <summary>
        /// The progress.
        /// </summary>
        private int progress;

        /// <summary>
        /// Initializes a new instance of the <see cref="ShellViewModel"/> class.
        /// </summary>
        public ShellViewModel()
        {
            try
            {
                this.moduleManager = ServiceLocator.Current.GetService(typeof(IModuleManager)) as IModuleManager;
                this.moduleCatalog = ServiceLocator.Current.GetService(typeof(IModuleCatalog)) as IModuleCatalog;

                this.loadModuleCommand = new DelegateCommand<object>(
                o => this.LoadModule_Execute(o),
                o => this.LoadModule_CanExecute(o));
            }
            catch (System.Exception)
            {
                // Ignore
            }
        }

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets the progress.
        /// </summary>
        /// <value>
        /// The progress.
        /// </value>
        public int Progress
        {
            get
            {
                return this.progress;
            }

            set
            {
                this.progress = value;
                this.PropertyChanged.Raise(this, o => o.Progress);
            }
        }

        /// <summary>
        /// Gets the load module command.
        /// </summary>
        public ICommand LoadModuleCommand
        {
            get
            {
                return this.loadModuleCommand;
            }
        }

        /// <summary>
        /// Load Module Command Action.
        /// </summary>
        /// <param name="parameter">The parameter.</param>
        private void LoadModule_Execute(object parameter)
        {
            (this.moduleManager as MefModuleManager).ModuleDownloadProgressChanged += new System.EventHandler<ModuleDownloadProgressChangedEventArgs>(this.ShellViewModel_ModuleDownloadProgressChanged);
            this.moduleManager.LoadModule("Prism.Sample.OnDemandModule");
            this.loadModuleCommand.RaiseCanExecuteChanged();
        }

        /// <summary>
        /// Handles the ModuleDownloadProgressChanged event of the ShellViewModel.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Microsoft.Practices.Prism.Modularity.ModuleDownloadProgressChangedEventArgs"/> instance containing the event data.</param>
        private void ShellViewModel_ModuleDownloadProgressChanged(object sender, ModuleDownloadProgressChangedEventArgs e)
        {
            this.Progress = e.ProgressPercentage;
        }

        /// <summary>
        /// Load Module Command Logic.
        /// </summary>
        /// <param name="parameter">The parameter.</param>
        /// <returns>Whether or not the load module action can execute.</returns>
        private bool LoadModule_CanExecute(object parameter)
        {
            int query = (from m in this.moduleCatalog.Modules
                         where m.ModuleName == "Prism.Sample.OnDemandModule" && m.State == ModuleState.NotStarted
                         select m).Count();
            return query > 0;
        }
    }
}
